// Copyright 2024 Pierre Lairez
// SPDX-License-Identifier: GPL-3.0-or-later

pub mod complex;
mod f64extra;
pub mod real;

use std::arch::x86_64::*;

pub type Rif = real::Interval;
pub type Interval = real::Interval;
pub type Cif = complex::Interval;
pub type ComplexInterval = complex::Interval;

#[inline(always)]
pub fn with_rounding_up<T, F>(f: F) -> T
where
    F: FnOnce() -> T,
{
    #[allow(deprecated)]
    unsafe {
        let csr = _mm_getcsr();
        // _mm_setcsr(_MM_ROUND_UP | _MM_MASK_MASK);
        // let ret = f();
        // _mm_setcsr(csr);
        // ret
        // f()
        let newcsr = _MM_ROUND_UP | _MM_MASK_MASK;
        if csr & newcsr != newcsr {
            _mm_setcsr(_MM_ROUND_UP | _MM_MASK_MASK);
        }

        let ret = f();

        if csr & newcsr != newcsr {
            _mm_setcsr(csr);
        }
        ret
    }
}
