// Copyright 2024 Pierre Lairez
// SPDX-License-Identifier: GPL-3.0-or-later

use std::{path::PathBuf, time::Instant};

use algpath::{
    reckless::Cif,
    traits::{
        system::RecklessSys,
        track::{
            find_moore_box::{FindMooreBox, RecklessFindMooreBox},
            RecklessTrack5, Track,
        },
    },
};

#[test]
fn track() {
    let path = PathBuf::from("data/2-2_1.json");
    let mut conf = algpath::cli::Configuration {
        path: Some(path),
        ..Default::default()
    };

    let start = Instant::now();
    for homogenize in [false] {
        for horner in [false] {
            conf.homogenize = homogenize;
            conf.horner = horner;

            let mut input_problem = algpath::cli::Problem::new_from_input(&conf).unwrap();
            let exact_sys = input_problem.build_parametric_system(&conf).unwrap();

            let path = &input_problem.path;
            let mut sys: RecklessSys = exact_sys.into();

            for point in &input_problem.fiber {
                let proj_point = algpath::projective::ProjectivePoint {
                    coord: point.clone().iter().map(|c| c.into()).collect::<Vec<Cif>>().into(),
                    chart: 0,
                };

                let mut mbox = RecklessFindMooreBox::find_moore_box(
                    &mut sys,
                    &path[0].clone().iter().map(|c| c.into()).collect::<Vec<Cif>>().into(),
                    proj_point,
                )
                .unwrap();

                let _tr = RecklessTrack5::track(&mut sys, &mut mbox).unwrap();
                dbg!(_tr.len());
            }
        }
    }
    let duration = start.elapsed();
    dbg!(duration);
}
